﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Command {
  [StorableClass]
  public class AddRowCommand : ColumnGroupCommandBase {
    [Storable]
    private IComparable[] newRow;

    private int position;
    private ICollection<int> oldSortedColumnIndices;
    private IEnumerable<SortOrder> oldSortOrder;

    private AddRowCommand()
      : base(null, string.Empty) {
    }

    public AddRowCommand(DataSet dataSet, string columnGroupName, IComparable[] row)
      : base(dataSet, columnGroupName) {
      this.newRow = row;
    }

    public override void Execute() {
      base.Execute();
      oldSortedColumnIndices = new List<int>(ColumnGroup.SortedColumnIndexes);
      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      ColumnGroup.AddRow(newRow);
      this.position = ColumnGroup.RowCount - 1;
      ColumnGroup.ResetSorting();
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      ColumnGroup.DeleteRow(position);
      ColumnGroup.SortOrdersForColumns = oldSortOrder;
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      oldSortedColumnIndices = null;
      oldSortOrder = null;
      this.position = -1;
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override string Description {
      get { return "New row added"; }
    }
  }
}
