﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Xml;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Command {
  [StorableClass]
  public class ChangeValuesToNullCommand : ColumnGroupCommandBase {
    [Storable]
    private Dictionary<Point, IComparable> cellValues;

    private ICollection<int> oldSortedColumnIndexes;
    private IEnumerable<SortOrder> oldSortOrder;

    private ChangeValuesToNullCommand()
      : base(null, string.Empty) {
    }

    public ChangeValuesToNullCommand(DataSet dataSet, string columnGroupName, IEnumerable<Point> cellIndexes)
      : base(dataSet, columnGroupName) {
      cellValues = new Dictionary<Point, IComparable>();
      foreach (Point cellIndex in cellIndexes)
        cellValues.Add(cellIndex, null);
    }

    public override void Execute() {
      base.Execute();
      foreach (Point cellIndex in cellValues.Keys.ToList()) {
        cellValues[cellIndex] = ColumnGroup.GetColumn(cellIndex.X).GetValue(cellIndex.Y);
        ColumnGroup.GetColumn(cellIndex.X).ChangeValue(cellIndex.Y, null);
      }
      oldSortedColumnIndexes = new List<int>(ColumnGroup.SortedColumnIndexes);
      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      foreach (Point cellIndex in cellValues.Keys.ToList()) {
        ColumnGroup.GetColumn(cellIndex.X).ChangeValueOrNull(cellIndex.Y, cellValues[cellIndex]);
        cellValues[cellIndex] = null;
      }
      ColumnGroup.SortOrdersForColumns = oldSortOrder;
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndexes;
      oldSortedColumnIndexes = null;
      oldSortOrder = null;
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override string Description {
      get { return "Column values deleted"; }
    }
  }
}
