﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Command {
  [StorableClass]
  public class DeleteColumnCommand : ColumnGroupCommandWithAffectedColumnsBase {
    private List<ColumnBase> deletedColumns;
    private List<int> oldSortedColumnIndices;

    private DeleteColumnCommand()
      : base(null, string.Empty, null) {
      deletedColumns = new List<ColumnBase>();
    }

    public DeleteColumnCommand(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      deletedColumns = new List<ColumnBase>();
    }

    public override void Execute() {
      base.Execute();
      this.oldSortedColumnIndices = new List<int>(ColumnGroup.SortedColumnIndexes);
      ColumnBase column;
      foreach (int columnIndex in AffectedColumns.Reverse()) {
        column = ColumnGroup.GetColumn(columnIndex);
        deletedColumns.Insert(0, column);
        ColumnGroup.RemoveColumn(column);
        if (column.SortOrder != SortOrder.None)
          ColumnGroup.SortedColumnIndexes.Remove(columnIndex);
      }
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      for (int i = 0; i < AffectedColumns.Length; i++)
        ColumnGroup.InsertColumn(AffectedColumns[i], deletedColumns[i]);

      deletedColumns.Clear();
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      oldSortedColumnIndices = null;
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override string Description {
      get { return "Columns deleted"; }
    }
  }
}
