﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Data.Command {
  [StorableClass]
  public class InsertRowCommand : ColumnGroupCommandBase {
    [Storable]
    private int position;

    private ICollection<int> oldSortedColumnIndices;
    private IEnumerable<SortOrder> oldSortOrder;

    private InsertRowCommand()
      : base(null, string.Empty) {
    }

    public InsertRowCommand(DataSet dataSet, string columnGroupName, int position)
      : base(dataSet, columnGroupName) {
      this.position = position;
    }

    public override void Execute() {
      base.Execute();
      ColumnGroup.InsertRow(position, ColumnGroup.GetEmptyRow());
      oldSortedColumnIndices = new List<int>(ColumnGroup.SortedColumnIndexes);
      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override void UndoExecute() {
      base.UndoExecute();
      ColumnGroup.DeleteRow(position);
      ColumnGroup.SortOrdersForColumns = oldSortOrder;
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndices;
      oldSortedColumnIndices = null;
      oldSortOrder = null;
      ColumnGroup.FireChanged();
      this.ColumnGroup = null;
    }

    public override string Description {
      get { return "Empty row inserted"; }
    }
  }
}
